from pathlib import Path

class GtImgAlligner:
    def __init__(self, img_ids) -> None:
        image_paths = []
        image_urls = []
        gt_paths = []
        gt_auto_paths = []
    
    def imgfname2jsonfname(img_id):
        pass
    
    def imgfname2jsonfname(img_id):
        pass

    def imgfname2jsonfname(img_id):
        pass
    
